package com.uas.account;

import com.uas.sso.SSOConfig;

/**
 * 账户中心配置
 * 
 * @author yingp
 * 
 */
public class AccountConfig extends SSOConfig {

	public static final String ACCOUNT_CENTER = "sso";

	/**
	 * 无需绑定到任何企业的个人账号，默认绑定到-1
	 */
	public static final String DEFAULT_SPACE_ID = "-1";

	/**
	 * 账户中心的接口，App注册企业信息到账户中心
	 * 
	 * <pre>
	 * 账户中心保存的资料有限，因此App里面也需要有自定义的企业信息表来存放业务字段；
	 * App里面最好是在自己系统保存成功后，再调用账户中心的接口，即使出错也可回滚。
	 * </pre>
	 */
	public static String getSpaceSaveUrl() {
		return getInstance().getProperty("account.us.save.url");
	}

	/**
	 * 账户中心的接口，App注册用户信息到账户中心
	 * 
	 * <pre>
	 * 账户中心保存的资料有限，因此App里面也需要有自定义的用户信息表来存放业务字段；
	 * App里面最好是在自己系统保存成功后，再调用账户中心的接口，即使出错也可回滚。
	 * </pre>
	 */
	public static String getUserSaveUrl() {
		return getInstance().getProperty("account.user.save.url");
	}

	/**
	 * 账户中心的接口，供商务平台和ERP系统查询自己当期企业的合作伙伴
	 */
	public static String getEnPartnersUrl() {
		return getInstance().getProperty("account.user.getPartners.url");
	}

	/**
	 * 账户中心接口，跳转到企业圈
	 */
	public static String getContactPageUrl() {
		return getInstance().getProperty("account.user.getContactPage.url");
	}
}
